#!/bin/bash
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2010 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.

# Return error if not root when running this script
if [ `whoami` != 'root' ]; then
    echo "Must have root level authority to run this script"
    exit 99
fi

# config options
unset ACTION MODE NETWORK VLAN_ID VERBOSE

# defaults
ACTION_ADD="add"
ACTION_CHECK="check"
ACTION_REMOVE="remove"
ACTION=$ACTION_ADD

MODE_TEMP="temp"
MODE_STATIC="static"
MODE_DYNAMIC="dynamic"
MODE=$MODE_DYNAMIC

# error return codes
RC_SUCCESS=0
RC_USAGE=1
RC_VLAN_ALREADY_EXISTS=2
RC_VLAN_DOES_NOT_EXIST=3
RC_NO_VLANS_EXIST=4
RC_IFCFG_FILE_EXISTS=5
RC_INVALID_INTERFACE_NAME=6
RC_INVALID_VLAN_ID=7

# other constants
PROG=`basename $0`
RHEL_IFCFG_PATH="/etc/sysconfig/network-scripts"

### Utility Functions
function debug {
    if [ -z $VERBOSE ]; then
        return
    fi
    echo "DEBUG: $@" 1>&2
}

function err {
    echo "ERROR: $@" 1>&2
}

function err_exit {
    local rc=$1
    shift
    err $@
    exit $rc
}

function dsp_errmsg {
    local rc_map=( )
    rc_map[RC_USAGE]="Invalid parameter or -h was specified"
    rc_map[RC_VLAN_ALREADY_EXISTS]="The specified vlan id already exists"
    rc_map[RC_VLAN_DOES_NOT_EXIST]="The specified vlan id does not exist"
    rc_map[RC_NO_VLANS_EXIST]="There are no vlans defined on this system"
    rc_map[RC_IFCFG_FILE_EXISTS]="The ifcfg-* file already exists, add aborted."
    rc_map[RC_INVALID_INTERFACE_NAME]="The interface name specified is not valid."
    rc_map[RC_INVALID_VLAN_ID]="The VLAN id specified is not valid."

    for rc in $@; do
        printf "%s: %s\n" $rc "${rc_map[$rc]}"
    done
    exit $RC_SUCCESS
}

function usage {
    if [ "$1" ]; then
        err $1
        echo ""
    fi

    cat <<EOF
USAGE :: $PROG  [-a|c|r] [-t|s|d] [-v]  <interface>  <vlan id>
         $PROG  [-hl]
         $PROG  [-e]  <Error num 1>  ... <Error num N>

    Required Parameters:
    ---------------------------------------------------------------------------
    <interface>     eth0, eth1
    <vlan id>       vlan id from 1 to 4095

    Action Parameters (select only 1):
    ---------------------------------------------------------------------------
    -a|--add        Add the VLAN id to the interface  (DEFAULT)
    -c|--check      Check if the VLAN id is already on the interface
    -r|--remove     Remove the VLAN id from the interface

    Mode Parameters (select only 1):
    ---------------------------------------------------------------------------
    -t|--temp      Temporarily change.  Do not update static network files
    -s|--static    Update static network files and restart network service
    -d|--dynamic   Update network files and dynamic update network  (DEFAULT)

    Additional Optional Parameters:
    ---------------------------------------------------------------------------
    -h|--help      Display this message
    -v|--verbose   Output debug information to track execution

    -e|--errmsg    Get the human readable translation of the specified return
                   code. (Ignores all other options)
    -l|--list      Displays all vlans configured (Ignores all other options)

EOF
    exit $RC_USAGE
}

###end Utility Functions

### Action functions
function list_vlans {
    if [ -f /proc/net/vlan/config ]; then
        cat /proc/net/vlan/config
    else
        err_exit $RC_NO_VLANS_EXIST "There are no VLANs defined on this system"
    fi
    exit $RC_SUCCESS
}

function add_temp {
    echo "Adding $NETWORK.$VLAN_ID"

    # configure the interface and the bridge
    vconfig add $NETWORK $VLAN_ID
    brctl addbr br$NETWORK.$VLAN_ID
    brctl addif br$NETWORK.$VLAN_ID $NETWORK.$VLAN_ID

    # make sure new interface and bridge are both up
    ifconfig $NETWORK.$VLAN_ID up
    ifconfig br$NETWORK.$VLAN_ID up
}

function add_static {
    echo "Create network files for $NETWORK.$VLAN_ID"

    local interface_file="$RHEL_IFCFG_PATH/ifcfg-$NETWORK.$VLAN_ID"
    local bridge_file="$RHEL_IFCFG_PATH/ifcfg-br$NETWORK.$VLAN_ID"

    if [ -f $interface_file ]; then
        err_exit $RC_IFCFG_FILE_EXISTS "$interface_file already exists"
    fi

    if [ -f $bridge_file ]; then
        err_exit $RC_IFCFG_FILE_EXISTS "$bridge_file already exists"
    fi

    echo "DEVICE=$NETWORK.$VLAN_ID"     >  $interface_file
    echo "ONBOOT=yes"                   >> $interface_file
    echo "VLAN=yes"                     >> $interface_file
    echo "BRIDGE=br$NETWORK.$VLAN_ID"   >> $interface_file
    chmod 644  $interface_file

    echo "DEVICE=br$NETWORK.$VLAN_ID"   >  $bridge_file
    echo "TYPE=Bridge"                  >> $bridge_file
    echo "DELAY=0"                      >> $bridge_file
    echo "ONBOOT=yes"                   >> $bridge_file
    chmod 644  $bridge_file

    # If the mode is "static", then restart network
    if [ $MODE = $MODE_STATIC ]; then
        echo "Restarting the network"
        /etc/init.d/network restart
    fi
}

function add_dynamic {
    add_static
    add_temp
}

function remove_temp {
    echo "Remove $NETWORK.$VLAN_ID"

    # shut down the interface and the bridge
    ifconfig br$NETWORK.$VLAN_ID down
    ifconfig $NETWORK.$VLAN_ID down

    # unconfigure the interface and the bridge
    brctl delif br$NETWORK.$VLAN_ID $NETWORK.$VLAN_ID
    brctl delbr br$NETWORK.$VLAN_ID
    vconfig rem $NETWORK.$VLAN_ID
}

function remove_static {

    local interface_file="$RHEL_IFCFG_PATH/ifcfg-$NETWORK.$VLAN_ID"
    local bridge_file="$RHEL_IFCFG_PATH/ifcfg-br$NETWORK.$VLAN_ID"

    # If the mode is "static", then restart network
    if [ $MODE = $MODE_STATIC ]; then
        echo "Stopping the network"
        /etc/init.d/network stop
    fi

    echo "Delete the network files for $NETWORK.$VLAN_ID"
    rm -f $interface_file
    rm -f $bridge_file

    # If the mode is "static", then restart network
    if [ $MODE = $MODE_STATIC ]; then
        echo "Starting the network"
        /etc/init.d/network start
    fi
}

function remove_dynamic {
    remove_static
    remove_temp
}

###end Action functions

### Parameter parsing

# Usage flags
FLAGS="acrtsdvhel"
LONG_FLAGS="add,check,remove,temp,static,dynamic,verbose,help,errmsg,list"

args=`getopt -u -n $PROG -l $LONG_FLAGS "$FLAGS" $@` || usage
set - $args

while [ $1 != "--" ]; do
    case $1 in
        -a|--add)     ACTION=$ACTION_ADD    ;;
        -c|--check)   ACTION=$ACTION_CHECK  ;;
        -r|--remove)  ACTION=$ACTION_REMOVE ;;

        -t|--temp)    MODE=$MODE_TEMP    ;;
        -s|--static)  MODE=$MODE_STATIC  ;;
        -d|--dynamic) MODE=$MODE_DYNAMIC ;;

        -h|--help)    usage ;;
        -v|--verbose) VERBOSE=1 ;;
        -e|--errmsg)  shift 2; dsp_errmsg $@ ;;
        -l|--list)    list_vlans ;;
        *) usage "Invalid option: $1 not recognized" ;;
    esac
    shift
done

shift # get rid of the -- marker

# Make sure correct number of parameters are passed in
if [ $# -lt 2 ]; then
    usage "Interface and vlan ID were not passed in"
fi
NETWORK=$1
VLAN_ID=$2

# Dump out the parameters we are working with
debug "Network=$NETWORK, Vlan id=$VLAN_ID, Action=$ACTION, Mode=$MODE"

ifconfig $NETWORK > /dev/null 2>&1
if [ $? != 0 ]; then
    err_exit $RC_INVALID_INTERFACE_NAME "The interface name, $NETWORK, is not valid"
fi

if [ "$VLAN_ID" -lt "1" -o "$VLAN_ID" -gt "4095" ]; then
    err_exit $RC_INVALID_VLAN_ID "The VLAN id, $VLAN_ID, is not valid"
fi

###end Parameter parsing

### Process action
if [ $ACTION = $ACTION_ADD ]; then
    debug "Adding $NETWORK.$VLAN_ID"
    if [ -f /proc/net/vlan/$NETWORK.$VLAN_ID ]; then
        err_exit $RC_VLAN_ALREADY_EXISTS "$NETWORK.$VLAN_ID already exists"
    fi
    case $MODE in
        $MODE_TEMP)     add_temp    ;;
        $MODE_STATIC)   add_static  ;;
        $MODE_DYNAMIC)  add_dynamic ;;
        *) usage "Invalid option: $MODE not recognized" ;;
    esac
elif [ $ACTION = $ACTION_REMOVE ]; then
    debug "Removing $NETWORK.$VLAN_ID"
    if [ ! -f /proc/net/vlan/$NETWORK.$VLAN_ID ]; then
        err_exit $RC_VLAN_DOES_NOT_EXIST "$NETWORK.$VLAN_ID does not exist"
    fi
    case $MODE in
        $MODE_TEMP)     remove_temp    ;;
        $MODE_STATIC)   remove_static  ;;
        $MODE_DYNAMIC)  remove_dynamic ;;
        *) usage "Invalid option: $MODE not recognized" ;;
    esac
elif [ $ACTION = $ACTION_CHECK ]; then
    debug "Checking the status of $NETWORK.$VLAN_ID"
    if [ ! -f /proc/net/vlan/$NETWORK.$VLAN_ID ]; then
        err_exit $RC_VLAN_DOES_NOT_EXIST "$NETWORK.$VLAN_ID does not exist"
    else
        cat /proc/net/vlan/$NETWORK.$VLAN_ID
    fi
fi
###end Process action

echo "Success"
exit $RC_SUCCESS
